@@ -1,22 +1,11 @@
-"""
-    sphinx.domains.math
-    ~~~~~~~~~~~~~~~~~~~
+"""The math domain."""
 
-    The math domain.
-
-    :copyright: Copyright 2007-2020 by the Sphinx team, see AUTHORS.
-    :license: BSD, see LICENSE for details.
-"""
-
-import warnings
-from typing import Any, Dict, Iterable, List, Tuple
+from typing import TYPE_CHECKING, Any, Dict, Iterable, List, Optional, Tuple
 
 from docutils import nodes
-from docutils.nodes import Element, Node, system_message
-from docutils.nodes import make_id
+from docutils.nodes import Element, Node, make_id, system_message
 
 from sphinx.addnodes import pending_xref
-from sphinx.deprecation import RemovedInSphinx40Warning
 from sphinx.domains import Domain
 from sphinx.environment import BuildEnvironment
 from sphinx.locale import __
@@ -24,8 +13,7 @@ from sphinx.roles import XRefRole
 from sphinx.util import logging
 from sphinx.util.nodes import make_refnode
 
-if False:
-    # For type annotation
+if TYPE_CHECKING:
     from sphinx.application import Sphinx
     from sphinx.builders import Builder
 
@@ -45,10 +33,10 @@ class MathDomain(Domain):
     name = 'math'
     label = 'mathematics'
 
-    initial_data = {
+    initial_data: Dict = {
         'objects': {},  # labelid -> (docname, eqno)
         'has_equations': {},  # docname -> bool
-    }  # type: Dict
+    }
     dangling_warnings = {
         'eq': 'equation not found: %(target)s',
     }
@@ -82,10 +70,10 @@ class MathDomain(Domain):
         def math_node(node: Node) -> bool:
             return isinstance(node, (nodes.math, nodes.math_block))
 
-        self.data['has_equations'][docname] = any(document.traverse(math_node))
+        self.data['has_equations'][docname] = any(document.findall(math_node))
 
     def clear_doc(self, docname: str) -> None:
-        for equation_id, (doc, eqno) in list(self.equations.items()):
+        for equation_id, (doc, _eqno) in list(self.equations.items()):
             if doc == docname:
                 del self.equations[equation_id]
 
@@ -101,7 +89,7 @@ class MathDomain(Domain):
 
     def resolve_xref(self, env: BuildEnvironment, fromdocname: str, builder: "Builder",
                      typ: str, target: str, node: pending_xref, contnode: Element
-                     ) -> Element:
+                     ) -> Optional[Element]:
         assert typ in ('eq', 'numref')
         docname, number = self.equations.get(target, (None, None))
         if docname:
@@ -140,26 +128,11 @@ class MathDomain(Domain):
     def get_objects(self) -> List:
         return []
 
-    def add_equation(self, env: BuildEnvironment, docname: str, labelid: str) -> int:
-        warnings.warn('MathDomain.add_equation() is deprecated.',
-                      RemovedInSphinx40Warning)
-        if labelid in self.equations:
-            path = env.doc2path(self.equations[labelid][0])
-            msg = __('duplicate label of equation %s, other instance in %s') % (labelid, path)
-            raise UserWarning(msg)
+    def has_equations(self, docname: str = None) -> bool:
+        if docname:
+            return self.data['has_equations'].get(docname, False)
         else:
-            eqno = self.get_next_equation_number(docname)
-            self.equations[labelid] = (docname, eqno)
-            return eqno
-
-    def get_next_equation_number(self, docname: str) -> int:
-        warnings.warn('MathDomain.get_next_equation_number() is deprecated.',
-                      RemovedInSphinx40Warning)
-        targets = [eq for eq in self.equations.values() if eq[0] == docname]
-        return len(targets) + 1
-
-    def has_equations(self) -> bool:
-        return any(self.data['has_equations'].values())
+            return any(self.data['has_equations'].values())
 
 
 def setup(app: "Sphinx") -> Dict[str, Any]:
