@@ -1,12 +1,4 @@
-"""
-    sphinx.writers.xml
-    ~~~~~~~~~~~~~~~~~~
-
-    Docutils-native XML and pseudo-XML writers.
-
-    :copyright: Copyright 2007-2021 by the Sphinx team, see AUTHORS.
-    :license: BSD, see LICENSE for details.
-"""
+"""Docutils-native XML and pseudo-XML writers."""
 
 from typing import Any
 
